/* Usage:

one variable case:

b = pdols1(y,x,p,icase);

input: y = dependent variable TxN matrix
       x = indepedent variable TxN matrix
       p = the number of leads and lags
       icase = 1 if no constant
             = 2 if constant only
             = 3 if constant + linear trend

two variable case:

b = pdols2(y,x,z,p,icase);

input: y = dependent variable TxN matrix
       x = indepedent variable TxN matrix
       z = indepedent variable TxN matrix
       p = the number of leads and lags
       icase = 1 if no constant
             = 2 if constant only
             = 3 if constant + linear trend

Three variable case:

b = pdols3(y,x,z,m,p,icase);

input: y = dependent variable TxN matrix
       x = indepedent variable TxN matrix
       z = indepedent variable TxN matrix
       m = indepedent variable TxN matrix
       p = the number of leads and lags
       icase = 1 if no constant
             = 2 if constant only
             = 3 if constant + linear trend                              

and so on... */

@==================================== Procedure Starts ==============================@


proc  pdols2(y,x,z,p,icase);
local i,n,j,dx,dz,y1,x1,z1,xq,vy,vx,vz,vv,be,t;
local sq1,sq2,x2,z2,y2,x3,y3,z3,www,rr,q,skk,sk1,sk2;
local skk1,tr01,skk2,tr02,wx,wy,wz,ww,bw1,tw1,bw2,tw2;
local bb,bs,qqs,bbs,rrs,bs1,bs2,ss1,ss2;
local v2,b2;
local sq3,sq4,x4,y4,z4,x5,y5,z5,sk3,sk4,r2,ww2,tr03,tr04,skk3,skk4;

n = cols(y); t = rows(y);

if t < 50; q = 2; endif;
if t > 50 and t <= 100; q = 3; endif;
if t > 100; q = 4; endif;

       y1 = y[2+p:t-p,.];
       x1 = x[2+p:t-p,.];
       z1 = z[2+p:t-p,.];

       dx = zeros(rows(x[2+p:t-p,.]),2*p+1);
       dz = zeros(rows(x[2+p:t-p,.]),2*p+1);


  i = 1;
  do while i <= n;

    j = 1;
    do while j <= 2*p+1;
       dx[.,j] = x[2*p+3-j:t+1-j,i] - x[2*p+2-j:t-j,i];
       dz[.,j] = z[2*p+3-j:t+1-j,i] - z[2*p+2-j:t-j,i];
    j = j + 1;
    endo;
       y1[.,i] = y[2+p:t-p,i];
       x1[.,i] = x[2+p:t-p,i];
       z1[.,i] = z[2+p:t-p,i];

  if icase == 1; xq = dx~dz; endif;
  if icase == 2; xq = ones(rows(dx),1)~dx~dz; endif;
  if icase == 3; xq = ones(rows(dx),1)~seqa(1,1,rows(dx))~dx~dz; endif;

       xq = xq*invpd(xq'xq)*xq';
     y1[.,i] = y1[.,i] - xq*y1[.,i];
     x1[.,i] = x1[.,i] - xq*x1[.,i];
     z1[.,i] = z1[.,i] - xq*z1[.,i];

  i = i + 1;
  endo;

  bb   = zeros(2,n);
  bs1  = zeros(2,n);
  bs2  = zeros(2,n);

  i = 1;
  do while i <= n;
    qqs = x1[.,i]~z1[.,i];
    bbs = invpd(qqs'qqs)*qqs'y1[.,i];
    rrs = y1[.,i] - qqs*bbs;
    ss1 = pdlong(rrs,q-1);
    ss2 = qspw(rrs);
    ss1 = ss1*invpd(qqs'qqs);
    ss2 = ss2*invpd(qqs'qqs);
    ss1 = sqrt(diag(ss1));
    ss2 = sqrt(diag(ss2));
    bb[.,i] = bbs;
    bs1[.,i] = ss1;
    bs2[.,i] = ss2;
  i = i + 1;
  endo;

     vy = vec(y1);
     vx = vec(x1);
     vz = vec(z1);
     vv = vx~vz;

     be = invpd(vv'vv)*vv'vy;

     y2 = y1 - meanc(y1');
     x2 = x1 - meanc(x1');
     z2 = z1 - meanc(z1');
     
     vy = vec(y2);
     vx = vec(x2);
     vz = vec(z2);
     v2 = vx~vz;
     
     b2 = invpd(v2'v2)*v2'vy;

   sq1 = 0;
   sq2 = 0;
   sq3 = 0;
   sq4 = 0;

    i = 1;
    do while i <= n;
     www = x1[.,i]~z1[.,i];
     ww2 = x2[.,i]~z2[.,i];
     rr  = y1[.,i] - x1[.,i]*be[1] - z1[.,i]*be[2];
     r2  = y2[.,i] - x2[.,i]*b2[1] - z2[.,i]*b2[2];

     sk1 = pdlong(rr,q);
     sk2 = qspw(rr);

     sk3 = pdlong(r2,q);
     sk4 = qspw(r2);

     sq1      = sq1 + sk1*(www'www);
     sq2      = sq2 + sk2*(www'www);

     sq3      = sq3 + sk3*(ww2'ww2);
     sq4      = sq4 + sk4*(ww2'ww2);

    i = i + 1;
    endo;


   skk  = invpd(vv'vv);
   skk1 = skk*sq1*skk;           tr01 = sqrt(diag(skk1));
   skk2 = skk*sq2*skk;           tr02 = sqrt(diag(skk2));

   skk  = invpd(v2'v2);
   skk3 = skk*sq3*skk;           tr03 = sqrt(diag(skk3));
   skk4 = skk*sq4*skk;           tr04 = sqrt(diag(skk4));

format /m1 /rd 8,3;
"";
"";
"";
"";
"";
"";
"______________________________________________________________________";
"      Reference: Cointegration Vector Estimation by Panel DOLS        ";
"         and Long-Run Money Demand                                    ";
"         Mark: Mark.1@osu.edu      Sul: d.SUL@Auckland.ac.nz          ";
"                     Version: 2.0  Date: Nov 26, 2002.                ";
"______________________________________________________________________";
"";
"";
"";
if icase == 2;
"Model: y(it) = c(i) + A1*x(it) + A2*z(it) + u(it) ";
"";
endif;

if icase == 3;
"Model: y(it) = c(i) + d(i)t + A1*x(it) + A2*z(it) + u(it) ";
endif;
"";
"";
"";
"";

" _______________________ ESTIMATION RESULTS __________________________";
"";
"";
"";
format /m1 /rd 7,3;
"__________________ Dynamic OLS for Single Equation ___________________";
"";
"Eq No.    A1     S.E.      S.E      A2     S.E.     S.E     ";
"                Param    Andrews          Param   Andrews   ";
"_______________________________________________________________________";
i = 1;
do while i <= n;
i~bb[1,i]~bs1[1,i]~bs2[1,i]~bb[2,i]~bs1[2,i]~bs2[2,i];
i = i + 1;
endo;
"_______________________________________________________________________";
"";
"";
"";
"____________________________ Pooled  OLS ______________________________";
"";
"";
"";
"             Ordinary            Common Time Effect   ";
"     PDOLS   S.E.    S.E    PDOLS    S.E.    S.E.     ";
"            Param  Andrews  Param   Param  Andrews     ";
"_______________________________________________________________________";
" A1";;be[1]~tr01[1]~tr02[1];;b2[1]~tr03[1]~tr04[1];
" A2";;be[2]~tr01[2]~tr02[2];;b2[2]~tr03[2]~tr04[2];
"_______________________________________________________________________";
"";
" S.E. Andrews : Standard error based on Andrews and Monahan's Pre-whitening method";
" S.E. Param   : Standard error based on parametric correction";

retp(be);
endp;

proc  pdols1(y,x,p,icase);
local i,n,j,dx,dz,y1,x1,z1,xq,vy,vx,vz,vv,be,t;
local sq1,sq2,x2,z2,y2,x3,y3,z3,www,rr,q,skk,sk1,sk2;
local skk1,tr01,skk2,tr02,wx,wy,wz,ww,bw1,tw1,bw2,tw2;
local bb,bs,qqs,bbs,rrs,bs1,bs2,ss1,ss2;
local sq3,sq4,x4,y4,z4,x5,y5,z5,sk3,sk4,r2,ww2,tr03,tr04,skk3,skk4;
local v2,b2;

n = cols(y); t = rows(y);

if t < 50; q = 2; endif;
if t > 50 and t <= 100; q = 3; endif;
if t > 100; q = 4; endif;

       y1 = y[2+p:t-p,.];
       x1 = x[2+p:t-p,.];

       dx = zeros(rows(x[2+p:t-p,.]),2*p+1);


  i = 1;
  do while i <= n;

    j = 1;
    do while j <= 2*p+1;
       dx[.,j] = x[2*p+3-j:t+1-j,i] - x[2*p+2-j:t-j,i];
    j = j + 1;
    endo;
       y1[.,i] = y[2+p:t-p,i];
       x1[.,i] = x[2+p:t-p,i];

  if icase == 1; xq = dx; endif;
  if icase == 2; xq = ones(rows(dx),1)~dx; endif;
  if icase == 3; xq = ones(rows(dx),1)~seqa(1,1,rows(dx))~dx; endif;

       xq = xq*invpd(xq'xq)*xq';
     y1[.,i] = y1[.,i] - xq*y1[.,i];
     x1[.,i] = x1[.,i] - xq*x1[.,i];

  i = i + 1;
  endo;

  bb   = zeros(1,n);
  bs1  = zeros(1,n);
  bs2  = zeros(1,n);
  i = 1;
  do while i <= n;
    qqs = x1[.,i];
    bbs = invpd(qqs'qqs)*qqs'y1[.,i];
    rrs = y1[.,i] - qqs*bbs;
    ss1 = pdlong(rrs,q-1);
    ss2 = qspw(rrs);
    ss1 = ss1*invpd(qqs'qqs);
    ss2 = ss2*invpd(qqs'qqs);
    ss1 = sqrt(diag(ss1));
    ss2 = sqrt(diag(ss2));
    bb[.,i] = bbs;
    bs1[.,i] = ss1;
    bs2[.,i] = ss2;
  i = i + 1;
  endo;


     vy = vec(y1);
     vx = vec(x1);
     vv = vx;

     be = invpd(vv'vv)*vv'vy;


     y2 = y1 - meanc(y1');
     x2 = x1 - meanc(x1');
     
     vy = vec(y2);
     vx = vec(x2);
     v2 = vx;
     
     b2 = invpd(v2'v2)*v2'vy;

   sq1 = 0;
   sq2 = 0;
   sq3 = 0;
   sq4 = 0;

    i = 1;
    do while i <= n;
     www = x1[.,i];
     ww2 = x2[.,i];
     rr  = y1[.,i] - x1[.,i]*be[1];
     r2  = y2[.,i] - x2[.,i]*b2[1];

     sk1 = pdlong(rr,q);
     sk2 = qspw(rr);

     sk3 = pdlong(r2,q);
     sk4 = qspw(r2);

     sq1      = sq1 + sk1*(www'www);
     sq2      = sq2 + sk2*(www'www);

     sq3      = sq3 + sk3*(ww2'ww2);
     sq4      = sq4 + sk4*(ww2'ww2);

    i = i + 1;
    endo;


   skk  = invpd(vv'vv);
   skk1 = skk*sq1*skk;           tr01 = sqrt(diag(skk1));
   skk2 = skk*sq2*skk;           tr02 = sqrt(diag(skk2));

   skk  = invpd(v2'v2);
   skk3 = skk*sq3*skk;           tr03 = sqrt(diag(skk3));
   skk4 = skk*sq4*skk;           tr04 = sqrt(diag(skk4));


format /m1 /rd 8,3;
"";
"";
"";
"";
"";
"";
"______________________________________________________________________";
"      Reference: Cointegration Vector Estimation by Panel DOLS        ";
"         and Long-Run Money Demand                                    ";
"         Mark: Mark.1@osu.edu      Sul: d.SUL@Auckland.ac.nz          ";
"                     Version: 2.0  Date: Nov 26, 2002.                ";
"______________________________________________________________________";
"";
"";
"";
if icase == 2;
"Model: y(it) = c(i) + A1*x(it) + u(it) ";
"";
endif;

if icase == 3;
"Model: y(it) = c(i) + d(i)t + A1*x(it) + u(it) ";
endif;
"";
"";
"";
"";

" _______________________ ESTIMATION RESULTS __________________________";

"";
"";
"";
format /m1 /rd 7,3;
"__________________ Dynamic OLS for Single Equation ___________________";
"";
"Eq No.     A1    S.E.     S.E    ";
"                Param   Andrews  ";
"_______________________________________________________________________";
i = 1;
do while i <= n;
i~bb[1,i]~bs1[1,i];
i = i + 1;
endo;
"_______________________________________________________________________";
"";
"";
"";
"____________________________ Pooled  OLS ______________________________";
"";
"";
"";
"             Ordinary            Common Time Effect   ";
"     PDOLS   S.E.    S.E    PDOLS    S.E.    S.E.     ";
"            Param  Andrews  Param   Param  Andrews     ";
"_______________________________________________________________________";
" A1";;be[1]~tr01[1]~tr02[1];;b2[1]~tr03[1]~tr04[1];
"_______________________________________________________________________";
"";
" S.E. Andrews : Standard error based on Andrews's Pre-whitening method";
" S.E. Param   : Standard error based on parametric correction";

retp(be);
endp;


proc  pdols5(y,x,z,m,w,q,p,icase);
local i,n,j,dx,dz,y1,x1,z1,xq,vy,vx,vz,vv,be,t;
local sq1,sq2,x2,z2,y2,x3,y3,z3,www,rr,s,skk,sk1,sk2;
local skk1,tr01,skk2,tr02,wx,wy,wz,ww,bw1,tw1,bw2,tw2;
local bb,bs,qqs,bbs,rrs,bs1,bs2,ss1,ss2;
local v2,b2;
local sq3,sq4,x4,y4,z4,x5,y5,z5,sk3,sk4,r2,ww2,tr03,tr04,skk3,skk4;
local m1,w1,q1,dm,dw,dq,vm,vw,vq,m2,w2,q2;

n = cols(y); t = rows(y);

if t < 50; s = 2; endif;
if t > 50 and t <= 100; s = 3; endif;
if t > 100; s = 4; endif;

       y1 = y[2+p:t-p,.];
       x1 = x[2+p:t-p,.];
       z1 = z[2+p:t-p,.];
       m1 = m[2+p:t-p,.];
       w1 = w[2+p:t-p,.];
       q1 = q[2+p:t-p,.];

       dx = zeros(rows(x[2+p:t-p,.]),2*p+1);
       dz = zeros(rows(x[2+p:t-p,.]),2*p+1);
       dm = zeros(rows(m[2+p:t-p,.]),2*p+1);
       dw = zeros(rows(w[2+p:t-p,.]),2*p+1);
       dq = zeros(rows(q[2+p:t-p,.]),2*p+1);


  i = 1;
  do while i <= n;

    j = 1;
    do while j <= 2*p+1;
       dx[.,j] = x[2*p+3-j:t+1-j,i] - x[2*p+2-j:t-j,i];
       dz[.,j] = z[2*p+3-j:t+1-j,i] - z[2*p+2-j:t-j,i];
       dm[.,j] = m[2*p+3-j:t+1-j,i] - m[2*p+2-j:t-j,i];
       dw[.,j] = w[2*p+3-j:t+1-j,i] - w[2*p+2-j:t-j,i];
       dq[.,j] = q[2*p+3-j:t+1-j,i] - q[2*p+2-j:t-j,i];
    j = j + 1;
    endo;
    
       y1[.,i] = y[2+p:t-p,i];
       x1[.,i] = x[2+p:t-p,i];
       z1[.,i] = z[2+p:t-p,i];
       m1[.,i] = m[2+p:t-p,i];
       w1[.,i] = w[2+p:t-p,i];
       q1[.,i] = q[2+p:t-p,i];

  if icase == 1; xq = dx~dz~dm~dw~dq; endif;
  if icase == 2; xq = ones(rows(dx),1)~dx~dz~dm~dw~dq; endif;
  if icase == 3; xq = ones(rows(dx),1)~seqa(1,1,rows(dx))~dx~dz~dm~dw~dq; endif;

       xq = xq*invpd(xq'xq)*xq';
     y1[.,i] = y1[.,i] - xq*y1[.,i];
     x1[.,i] = x1[.,i] - xq*x1[.,i];
     z1[.,i] = z1[.,i] - xq*z1[.,i];
     m1[.,i] = m1[.,i] - xq*m1[.,i];
     w1[.,i] = w1[.,i] - xq*w1[.,i];
     q1[.,i] = q1[.,i] - xq*q1[.,i];

  i = i + 1;
  endo;

  bb   = zeros(5,n);
  bs1  = zeros(5,n);
  bs2  = zeros(5,n);

  i = 1;
  do while i <= n;
    qqs = x1[.,i]~z1[.,i]~m1[.,i]~w1[.,i]~q1[.,i];
    bbs = invpd(qqs'qqs)*qqs'y1[.,i];
    rrs = y1[.,i] - qqs*bbs;
    ss1 = pdlong(rrs,s-1);
    ss2 = qspw(rrs);
    ss1 = ss1*invpd(qqs'qqs);
    ss2 = ss2*invpd(qqs'qqs);
    ss1 = sqrt(diag(ss1));
    ss2 = sqrt(diag(ss2));
    bb[.,i] = bbs;
    bs1[.,i] = ss1;
    bs2[.,i] = ss2;
  i = i + 1;
  endo;


     vy = vec(y1);
     vx = vec(x1);
     vz = vec(z1);
     vm = vec(m1);
     vw = vec(w1);
     vq = vec(q1);

     vv = vx~vz~vm~vw~vq;

     be = invpd(vv'vv)*vv'vy;

     y2 = y1 - meanc(y1');
     x2 = x1 - meanc(x1');
     z2 = z1 - meanc(z1');
     m2 = m1 - meanc(m1');
     w2 = w1 - meanc(w1');
     q2 = q1 - meanc(q1');
     
     vy = vec(y2);
     vx = vec(x2);
     vz = vec(z2);
     vm = vec(m2);
     vw = vec(w2);
     vq = vec(q2);
     
     v2 = vx~vz~vm~vw~vq;
     
     b2 = invpd(v2'v2)*v2'vy;

   sq1 = 0;
   sq2 = 0;
   sq3 = 0;
   sq4 = 0;

    i = 1;
    do while i <= n;
     www = x1[.,i]~z1[.,i]~m1[.,i]~w1[.,i]~q1[.,i];
     ww2 = x2[.,i]~z2[.,i]~m2[.,i]~w2[.,i]~q2[.,i];
     rr  = y1[.,i] - www*be;
     r2  = y2[.,i] - ww2*b2;

     sk1 = pdlong(rr,s);
     sk2 = qspw(rr);

     sk3 = pdlong(r2,s);
     sk4 = qspw(r2);

     sq1      = sq1 + sk1*(www'www);
     sq2      = sq2 + sk2*(www'www);

     sq3      = sq3 + sk3*(ww2'ww2);
     sq4      = sq4 + sk4*(ww2'ww2);

    i = i + 1;
    endo;


   skk  = invpd(vv'vv);
   skk1 = skk*sq1*skk;           tr01 = sqrt(diag(skk1));
   skk2 = skk*sq2*skk;           tr02 = sqrt(diag(skk2));

   skk  = invpd(v2'v2);
   skk3 = skk*sq3*skk;           tr03 = sqrt(diag(skk3));
   skk4 = skk*sq4*skk;           tr04 = sqrt(diag(skk4));

format /m1 /rd 8,3;
"";
"";
"";
"";
"";
"";
"______________________________________________________________________";
"      Reference: Cointegration Vector Estimation by Panel DOLS        ";
"         and Long-Run Money Demand                                    ";
"         Mark: Mark.1@osu.edu      Sul: d.SUL@Auckland.ac.nz          ";
"                     Version: 2.0  Date: Nov 26, 2002.                ";
"______________________________________________________________________";
"";
"";
"";
if icase == 2;
"Model: y(it) = c(i) + A1*x(it) + A2*z(it) + A3*m(it) + A4*w(it)  + A5*q(it) +  u(it) ";
"";
endif;

if icase == 3;
"Model: y(it) = c(i) + d(i)*t + A1*x(it) + A2*z(it) + A3*m(it) + A4*w(it)  + A5*q(it) +  u(it) ";
endif;
"";
"";
"";
"";

" _______________________ ESTIMATION RESULTS __________________________";
"";
"";
"";
format /m1 /rd 7,3;
"__________________ Dynamic OLS for Single Equation ___________________";
"";
"Eq No.    A1     S.E.      S.E      A2     S.E.     S.E     A3     S.E.     S.E      A4     S.E.     S.E    A5     S.E.     S.E ";
"                Param    Andrews          Param   Andrews         Param   Andrews         Param   Andrews        Param   Andrews ";
"_______________________________________________________________________";
i = 1;
do while i <= n;
i~bb[1,i]~bs1[1,i]~bs2[1,i]~bb[2,i]~bs1[2,i]~bs2[2,i]~bb[3,i]~bs1[3,i]~bs2[3,i]~bb[4,i]~bs1[4,i]~bs2[4,i]~bb[5,i]~bs1[5,i]~bs2[5,i];
i = i + 1;
endo;
"_______________________________________________________________________";
"";
"";
"";
"____________________________ Pooled  OLS ______________________________";
"";
"";
"";
"             Ordinary            Common Time Effect   ";
"     PDOLS   S.E.    S.E    PDOLS    S.E.    S.E.     ";
"            Param  Andrews  Param   Param  Andrews     ";
"_______________________________________________________________________";
" A1";;be[1]~tr01[1]~tr02[1];;b2[1]~tr03[1]~tr04[1];
" A2";;be[2]~tr01[2]~tr02[2];;b2[2]~tr03[2]~tr04[2];
" A3";;be[3]~tr01[3]~tr02[3];;b2[3]~tr03[3]~tr04[3];
" A4";;be[4]~tr01[4]~tr02[4];;b2[4]~tr03[4]~tr04[4];
" A5";;be[5]~tr01[5]~tr02[5];;b2[5]~tr03[5]~tr04[5];
"_______________________________________________________________________";
"";
" S.E. Andrews : Standard error based on Andrews and Monahan's Pre-whitening method";
" S.E. Param   : Standard error based on parametric correction";

retp(be);
endp;



proc  pdols4(y,x,z,m,w,p,icase);
local i,n,j,dx,dz,y1,x1,z1,xq,vy,vx,vz,vv,be,t;
local sq1,sq2,x2,z2,y2,x3,y3,z3,www,rr,s,skk,sk1,sk2;
local skk1,tr01,skk2,tr02,wx,wy,wz,ww,bw1,tw1,bw2,tw2;
local bb,bs,qqs,bbs,rrs,bs1,bs2,ss1,ss2;
local v2,b2;
local sq3,sq4,x4,y4,z4,x5,y5,z5,sk3,sk4,r2,ww2,tr03,tr04,skk3,skk4;
local m1,w1,q1,dm,dw,dq,vm,vw,vq,m2,w2,q2;

n = cols(y); t = rows(y);

if t < 50; s = 2; endif;
if t > 50 and t <= 100; s = 3; endif;
if t > 100; s = 4; endif;

       y1 = y[2+p:t-p,.];
       x1 = x[2+p:t-p,.];
       z1 = z[2+p:t-p,.];
       m1 = m[2+p:t-p,.];
       w1 = w[2+p:t-p,.];


       dx = zeros(rows(x[2+p:t-p,.]),2*p+1);
       dz = zeros(rows(x[2+p:t-p,.]),2*p+1);
       dm = zeros(rows(m[2+p:t-p,.]),2*p+1);
       dw = zeros(rows(w[2+p:t-p,.]),2*p+1);



  i = 1;
  do while i <= n;

    j = 1;
    do while j <= 2*p+1;
       dx[.,j] = x[2*p+3-j:t+1-j,i] - x[2*p+2-j:t-j,i];
       dz[.,j] = z[2*p+3-j:t+1-j,i] - z[2*p+2-j:t-j,i];
       dm[.,j] = m[2*p+3-j:t+1-j,i] - m[2*p+2-j:t-j,i];
       dw[.,j] = w[2*p+3-j:t+1-j,i] - w[2*p+2-j:t-j,i];

    j = j + 1;
    endo;
    
       y1[.,i] = y[2+p:t-p,i];
       x1[.,i] = x[2+p:t-p,i];
       z1[.,i] = z[2+p:t-p,i];
       m1[.,i] = m[2+p:t-p,i];
       w1[.,i] = w[2+p:t-p,i];


  if icase == 1; xq = dx~dz~dm~dw; endif;
  if icase == 2; xq = ones(rows(dx),1)~dx~dz~dm~dw; endif;
  if icase == 3; xq = ones(rows(dx),1)~seqa(1,1,rows(dx))~dx~dz~dm~dw; endif;

       xq = xq*invpd(xq'xq)*xq';
     y1[.,i] = y1[.,i] - xq*y1[.,i];
     x1[.,i] = x1[.,i] - xq*x1[.,i];
     z1[.,i] = z1[.,i] - xq*z1[.,i];
     m1[.,i] = m1[.,i] - xq*m1[.,i];
     w1[.,i] = w1[.,i] - xq*w1[.,i];

  i = i + 1;
  endo;

  bb   = zeros(4,n);
  bs1  = zeros(4,n);
  bs2  = zeros(4,n);

  i = 1;
  do while i <= n;
    qqs = x1[.,i]~z1[.,i]~m1[.,i]~w1[.,i];
    bbs = invpd(qqs'qqs)*qqs'y1[.,i];
    rrs = y1[.,i] - qqs*bbs;
    ss1 = pdlong(rrs,s-1);
    ss2 = qspw(rrs);
    ss1 = ss1*invpd(qqs'qqs);
    ss2 = ss2*invpd(qqs'qqs);
    ss1 = sqrt(diag(ss1));
    ss2 = sqrt(diag(ss2));
    bb[.,i] = bbs;
    bs1[.,i] = ss1;
    bs2[.,i] = ss2;
  i = i + 1;
  endo;


     vy = vec(y1);
     vx = vec(x1);
     vz = vec(z1);
     vm = vec(m1);
     vw = vec(w1);


     vv = vx~vz~vm~vw;

     be = invpd(vv'vv)*vv'vy;

     y2 = y1 - meanc(y1');
     x2 = x1 - meanc(x1');
     z2 = z1 - meanc(z1');
     m2 = m1 - meanc(m1');
     w2 = w1 - meanc(w1');

     
     vy = vec(y2);
     vx = vec(x2);
     vz = vec(z2);
     vm = vec(m2);
     vw = vec(w2);
     
     v2 = vx~vz~vm~vw;
     
     b2 = invpd(v2'v2)*v2'vy;

   sq1 = 0;
   sq2 = 0;
   sq3 = 0;
   sq4 = 0;

    i = 1;
    do while i <= n;
     www = x1[.,i]~z1[.,i]~m1[.,i]~w1[.,i];
     ww2 = x2[.,i]~z2[.,i]~m2[.,i]~w2[.,i];
     rr  = y1[.,i] - www*be;
     r2  = y2[.,i] - ww2*b2;

     sk1 = pdlong(rr,s);
     sk2 = qspw(rr);

     sk3 = pdlong(r2,s);
     sk4 = qspw(r2);

     sq1      = sq1 + sk1*(www'www);
     sq2      = sq2 + sk2*(www'www);

     sq3      = sq3 + sk3*(ww2'ww2);
     sq4      = sq4 + sk4*(ww2'ww2);

    i = i + 1;
    endo;


   skk  = invpd(vv'vv);
   skk1 = skk*sq1*skk;           tr01 = sqrt(diag(skk1));
   skk2 = skk*sq2*skk;           tr02 = sqrt(diag(skk2));

   skk  = invpd(v2'v2);
   skk3 = skk*sq3*skk;           tr03 = sqrt(diag(skk3));
   skk4 = skk*sq4*skk;           tr04 = sqrt(diag(skk4));

format /m1 /rd 8,3;
"";
"";
"";
"";
"";
"";
"______________________________________________________________________";
"      Reference: Cointegration Vector Estimation by Panel DOLS        ";
"         and Long-Run Money Demand                                    ";
"         Mark: Mark.1@osu.edu      Sul: d.SUL@Auckland.ac.nz          ";
"                     Version: 2.0  Date: Nov 26, 2002.                ";
"______________________________________________________________________";
"";
"";
"";
if icase == 2;
"Model: y(it) = c(i) + A1*x(it) + A2*z(it) + A3*m(it) + A4*w(it)  +  u(it) ";
"";
endif;

if icase == 3;
"Model: y(it) = c(i) + d(i)*t + A1*x(it) + A2*z(it) + A3*m(it) + A4*w(it) +  u(it) ";
endif;
"";
"";
"";
"";

" _______________________ ESTIMATION RESULTS __________________________";
"";
"";
"";
format /m1 /rd 7,3;
"__________________ Dynamic OLS for Single Equation ___________________";
"";
"Eq No.    A1     S.E.      S.E      A2     S.E.     S.E     A3     S.E.     S.E      A4     S.E.     S.E     ";
"                Param    Andrews          Param   Andrews         Param   Andrews         Param   Andrews   ";
"_______________________________________________________________________";
i = 1;
do while i <= n;
i~bb[1,i]~bs1[1,i]~bs2[1,i]~bb[2,i]~bs1[2,i]~bs2[2,i]~bb[3,i]~bs1[3,i]~bs2[3,i]~bb[4,i]~bs1[4,i]~bs2[4,i];
i = i + 1;
endo;
"_______________________________________________________________________";
"";
"";
"";
"____________________________ Pooled  OLS ______________________________";
"";
"";
"";
"             Ordinary            Common Time Effect   ";
"     PDOLS   S.E.    S.E    PDOLS    S.E.    S.E.     ";
"            Param  Andrews  Param   Param  Andrews     ";
"_______________________________________________________________________";
" A1";;be[1]~tr01[1]~tr02[1];;b2[1]~tr03[1]~tr04[1];
" A2";;be[2]~tr01[2]~tr02[2];;b2[2]~tr03[2]~tr04[2];
" A3";;be[3]~tr01[3]~tr02[3];;b2[3]~tr03[3]~tr04[3];
" A4";;be[4]~tr01[4]~tr02[4];;b2[4]~tr03[4]~tr04[4];
"_______________________________________________________________________";
"";
" S.E. Andrews : Standard error based on Andrews and Monahan's Pre-whitening method";
" S.E. Param   : Standard error based on parametric correction";

retp(be);
endp;



proc  pdols3(y,x,z,m,p,icase);
local i,n,j,dx,dz,y1,x1,z1,xq,vy,vx,vz,vv,be,t;
local sq1,sq2,x2,z2,y2,x3,y3,z3,www,rr,s,skk,sk1,sk2;
local skk1,tr01,skk2,tr02,wx,wy,wz,ww,bw1,tw1,bw2,tw2;
local bb,bs,qqs,bbs,rrs,bs1,bs2,ss1,ss2;
local v2,b2;
local sq3,sq4,x4,y4,z4,x5,y5,z5,sk3,sk4,r2,ww2,tr03,tr04,skk3,skk4;
local m1,w1,q1,dm,dw,dq,vm,vw,vq,m2,w2,q2;

n = cols(y); t = rows(y);

if t < 50; s = 2; endif;
if t > 50 and t <= 100; s = 3; endif;
if t > 100; s = 4; endif;

       y1 = y[2+p:t-p,.];
       x1 = x[2+p:t-p,.];
       z1 = z[2+p:t-p,.];
       m1 = m[2+p:t-p,.];


       dx = zeros(rows(x[2+p:t-p,.]),2*p+1);
       dz = zeros(rows(x[2+p:t-p,.]),2*p+1);
       dm = zeros(rows(m[2+p:t-p,.]),2*p+1);

  i = 1;
  do while i <= n;

    j = 1;
    do while j <= 2*p+1;
       dx[.,j] = x[2*p+3-j:t+1-j,i] - x[2*p+2-j:t-j,i];
       dz[.,j] = z[2*p+3-j:t+1-j,i] - z[2*p+2-j:t-j,i];
       dm[.,j] = m[2*p+3-j:t+1-j,i] - m[2*p+2-j:t-j,i];

    j = j + 1;
    endo;
    
       y1[.,i] = y[2+p:t-p,i];
       x1[.,i] = x[2+p:t-p,i];
       z1[.,i] = z[2+p:t-p,i];
       m1[.,i] = m[2+p:t-p,i];

  if icase == 1; xq = dx~dz~dm; endif;
  if icase == 2; xq = ones(rows(dx),1)~dx~dz~dm; endif;
  if icase == 3; xq = ones(rows(dx),1)~seqa(1,1,rows(dx))~dx~dz~dm; endif;

       xq = xq*invpd(xq'xq)*xq';
     y1[.,i] = y1[.,i] - xq*y1[.,i];
     x1[.,i] = x1[.,i] - xq*x1[.,i];
     z1[.,i] = z1[.,i] - xq*z1[.,i];
     m1[.,i] = m1[.,i] - xq*m1[.,i];

  i = i + 1;
  endo;

  bb   = zeros(3,n);
  bs1  = zeros(3,n);
  bs2  = zeros(3,n);

  i = 1;
  do while i <= n;
    qqs = x1[.,i]~z1[.,i]~m1[.,i];
    bbs = invpd(qqs'qqs)*qqs'y1[.,i];
    rrs = y1[.,i] - qqs*bbs;
    ss1 = pdlong(rrs,s-1);
    ss2 = qspw(rrs);
    ss1 = ss1*invpd(qqs'qqs);
    ss2 = ss2*invpd(qqs'qqs);
    ss1 = sqrt(diag(ss1));
    ss2 = sqrt(diag(ss2));
    bb[.,i] = bbs;
    bs1[.,i] = ss1;
    bs2[.,i] = ss2;
  i = i + 1;
  endo;


     vy = vec(y1);
     vx = vec(x1);
     vz = vec(z1);
     vm = vec(m1);


     vv = vx~vz~vm;

     be = invpd(vv'vv)*vv'vy;

     y2 = y1 - meanc(y1');
     x2 = x1 - meanc(x1');
     z2 = z1 - meanc(z1');
     m2 = m1 - meanc(m1');

     
     vy = vec(y2);
     vx = vec(x2);
     vz = vec(z2);
     vm = vec(m2);
     
     v2 = vx~vz~vm;
     
     b2 = invpd(v2'v2)*v2'vy;

   sq1 = 0;
   sq2 = 0;
   sq3 = 0;
   sq4 = 0;

    i = 1;
    do while i <= n;
     www = x1[.,i]~z1[.,i]~m1[.,i];
     ww2 = x2[.,i]~z2[.,i]~m2[.,i];
     rr  = y1[.,i] - www*be;
     r2  = y2[.,i] - ww2*b2;

     sk1 = pdlong(rr,s);
     sk2 = qspw(rr);

     sk3 = pdlong(r2,s);
     sk4 = qspw(r2);

     sq1      = sq1 + sk1*(www'www);
     sq2      = sq2 + sk2*(www'www);

     sq3      = sq3 + sk3*(ww2'ww2);
     sq4      = sq4 + sk4*(ww2'ww2);

    i = i + 1;
    endo;


   skk  = invpd(vv'vv);
   skk1 = skk*sq1*skk;           tr01 = sqrt(diag(skk1));
   skk2 = skk*sq2*skk;           tr02 = sqrt(diag(skk2));

   skk  = invpd(v2'v2);
   skk3 = skk*sq3*skk;           tr03 = sqrt(diag(skk3));
   skk4 = skk*sq4*skk;           tr04 = sqrt(diag(skk4));

format /m1 /rd 8,3;
"";
"";
"";
"";
"";
"";
"______________________________________________________________________";
"      Reference: Cointegration Vector Estimation by Panel DOLS        ";
"         and Long-Run Money Demand                                    ";
"         Mark: Mark.1@osu.edu      Sul: d.SUL@Auckland.ac.nz          ";
"                     Version: 2.0  Date: Nov 26, 2002.                ";
"______________________________________________________________________";
"";
"";
"";
if icase == 2;
"Model: y(it) = c(i) + A1*x(it) + A2*z(it) + A3*m(it) +  u(it) ";
"";
endif;

if icase == 3;
"Model: y(it) = c(i) + d(i)*t + A1*x(it) + A2*z(it) + A3*m(it) +  u(it) ";
endif;
"";
"";
"";
"";

" _______________________ ESTIMATION RESULTS __________________________";
"";
"";
"";
format /m1 /rd 7,3;
"__________________ Dynamic OLS for Single Equation ___________________";
"";
"Eq No.    A1     S.E.      S.E      A2     S.E.     S.E     A3     S.E.     S.E      ";
"                Param    Andrews          Param   Andrews         Param   Andrews  ";
"_______________________________________________________________________";
i = 1;
do while i <= n;
i~bb[1,i]~bs1[1,i]~bs2[1,i]~bb[2,i]~bs1[2,i]~bs2[2,i]~bb[3,i]~bs1[3,i]~bs2[3,i];
i = i + 1;
endo;
"_______________________________________________________________________";
"";
"";
"";
"____________________________ Pooled  OLS ______________________________";
"";
"";
"";
"             Ordinary            Common Time Effect   ";
"     PDOLS   S.E.    S.E    PDOLS    S.E.    S.E.     ";
"            Param  Andrews  Param   Param  Andrews     ";
"_______________________________________________________________________";
" A1";;be[1]~tr01[1]~tr02[1];;b2[1]~tr03[1]~tr04[1];
" A2";;be[2]~tr01[2]~tr02[2];;b2[2]~tr03[2]~tr04[2];
" A3";;be[3]~tr01[3]~tr02[3];;b2[3]~tr03[3]~tr04[3];
"_______________________________________________________________________";
"";
" S.E. Andrews : Standard error based on Andrews and Monahan's Pre-whitening method";
" S.E. Param   : Standard error based on parametric correction";

retp(be);
endp;













@=================== Procedures for Long-run Estimators================@
@  Usage:                                                              @
@   se =  plong(x,p);  Paramteric method for long-run estimator with   @
@                      fixed lag length.                               @
@   se =  pdlong(x,p): Parametric method for long-run estimator with   @
@                      automatic lag selection based on Hall           @
@   se =  nwest(x,p):  Non-parametric method for long-run estimator    @
@                      Newey-West Batlett Kernel estimator             @
@                                                                      @
@   Input:                                                             @
@   x = (Tx1) vector                                                   @
@   p = lag length                                                     @
@   Output:                                                            @
@   se = long-run standard error                                       @
@!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!@



proc  plong(x,p);
local i,x1,j,yx,xx,b,resi,s_s,se,t_last,lambda,rio,t;
local ip,tl;
t = rows(x);
    x1 = zeros(t-p,p+1);
     j = 0;
      do until j > p;
       x1[.,j+1] = trimr(x,p-j,j);
      j = j + 1;
     endo;
  yx = x1[.,1]; xx= x1[.,2:cols(x1)];
  b = inv(xx'xx)*xx'yx;
  resi = yx - xx*b;
  s_s = resi'resi/(rows(x)-rows(b));
  lambda = s_s/(1-sumc(b))^2;

retp(lambda);
endp;

proc  pdlong(x,p);
local i,x1,j,y,xx,b,resi,s_s,se,t_last,lambda,rio,t;
local ip,tl,ss;
t = rows(x);
top:
ip = p;
if p == 0; lambda = sqrt((x'x)/(rows(x)-1)); retp(lambda); endif;
    x1 = zeros(t-ip,ip+1);

     j = 0;
      do until j > ip;
       x1[.,j+1] = trimr(x,ip-j,j);
      j = j + 1;
     endo;

  y = x1[.,1]; xx= x1[.,2:cols(x1)];
  b = invpd(xx'xx)*xx'y;
  resi = y - xx*b;
  s_s = resi'resi/(rows(y)-rows(b));
  ss  = diag(s_s*invpd(xx'xx));
  tl  = abs(b[rows(b)])./sqrt(ss[rows(ss)]);
  if tl > 1.96; goto final; endif;
  p = p - 1; goto top;
final:
 if sumc(b) > 1;
    "the sum of roots of error is greater than one. Reset maximum q again";
     endif;
lambda = s_s/(1-sumc(b))^2;
retp(lambda);
endp;

proc nwest(x,inlag);
local n,xom,xomm,i1,temp11;

n = rows(x);
   xom= (x'x)/n;
   xomm = 0;
    i1 = 1;
     do while i1 <= inlag;
      temp11 = sumc(x[1+i1:n-1,.].*x[1:n-1-i1,.]);
      xomm = (1-(i1/(inlag+1)))*(temp11/n) + xomm;
     i1 = i1 + 1;
    endo;

     xom = xom +2*xomm;
     
retp(xom);
endp;





proc qspw(x);
local t,uy,ux,b1,r1,a1,a2,mb,mq,kb,kq,ome,ome0,ssb,ssq,i,maxbq;

t = rows(x);
{uy,ux} = sauto(x,1);
b1 = invpd(ux'ux)*ux'uy;
if b1 >= 1; b1 = 0.999; endif;
r1 = uy - ux*b1;
a2 = (4*b1^2)/ ( (1-b1)^4 );
mq = int( 1.3221 * (a2*t)^(1/5) );
if mq == 0; mq = 1; endif;
if mq >= t-2; mq = t-4; endif;

maxbq = maxc(mq);

ome  = zeros(t-2,1);
ome0 = (r1'r1)./(t-1);
kq = ome;
i = 1;
do while i <= t-1-2;
ome[i] = (r1[i+1:t-1,.]'r1[1:t-1-i,.])./(t-1);
kq[i] = ( 25/(12*pi^2*(i/mq)^2) ) * ( (sin( 6*pi*(i/mq)/5 )/(6*pi*(i/mq)/5) )
       - cos(6*pi*(i/mq)/5 ) );
i = i + 1;
endo;
ssq = ome0 + 2*sumc(ome.*kq);

ssq = ssq/( (1-b1)^2);
retp(ssq);
endp;

proc (2) = sauto(x,p);
local z,i,y1,x1;
z = zeros(rows(x)-p,p+1);

i = 0;
do while i <= p;
z[.,i+1] = trimr(x,p-i,i);
i = i + 1;
endo;
y1 = z[.,1];
x1 = z[.,2:cols(z)];
retp(y1,x1);
endp;


